@@ -12,9 +12,12 @@ pub use self::kdf::{
     PBKDF2_OID, SCRYPT_OID,
 };
 
-use crate::{AlgorithmIdentifier, CryptoError, ObjectIdentifier};
+use crate::{AlgorithmIdentifier, CryptoError};
 use core::convert::{TryFrom, TryInto};
-use der::{Any, Decodable, Encodable, Encoder, Error, ErrorKind, Length, Message, OctetString};
+use der::{
+    asn1::{Any, ObjectIdentifier, OctetString},
+    Decodable, Encodable, Encoder, Error, ErrorKind, Length, Message,
+};
 
 #[cfg(all(feature = "alloc", feature = "pbes2"))]
 use alloc::vec::Vec;
@@ -23,10 +26,24 @@ use alloc::vec::Vec;
 /// Chaining (CBC) mode of operation.
 pub const AES_128_CBC_OID: ObjectIdentifier = ObjectIdentifier::new("2.16.840.1.101.3.4.1.2");
 
+/// 192-bit Advanced Encryption Standard (AES) algorithm with Cipher-Block
+/// Chaining (CBC) mode of operation.
+pub const AES_192_CBC_OID: ObjectIdentifier = ObjectIdentifier::new("2.16.840.1.101.3.4.1.22");
+
 /// 256-bit Advanced Encryption Standard (AES) algorithm with Cipher-Block
 /// Chaining (CBC) mode of operation.
 pub const AES_256_CBC_OID: ObjectIdentifier = ObjectIdentifier::new("2.16.840.1.101.3.4.1.42");
 
+/// DES operating in CBC mode
+#[cfg(feature = "des-insecure")]
+#[cfg_attr(docsrs, doc(cfg(feature = "des-insecure")))]
+pub const DES_CBC_OID: ObjectIdentifier = ObjectIdentifier::new("1.3.14.3.2.7");
+
+/// Triple DES operating in CBC mode
+#[cfg(feature = "3des")]
+#[cfg_attr(docsrs, doc(cfg(feature = "3des")))]
+pub const DES_EDE3_CBC_OID: ObjectIdentifier = ObjectIdentifier::new("1.2.840.113549.3.7");
+
 /// Password-Based Encryption Scheme 2 (PBES2) OID.
 ///
 /// <https://tools.ietf.org/html/rfc8018#section-6.2>
@@ -35,6 +52,10 @@ pub const PBES2_OID: ObjectIdentifier = ObjectIdentifier::new("1.2.840.113549.1.
 /// AES cipher block size
 const AES_BLOCK_SIZE: usize = 16;
 
+/// DES / Triple DES block size
+#[cfg(any(feature = "3des", feature = "des-insecure"))]
+const DES_BLOCK_SIZE: usize = 8;
+
 /// Password-Based Encryption Scheme 2 parameters as defined in [RFC 8018 Appendix A.4].
 ///
 /// ```text
@@ -86,17 +107,11 @@ impl<'a> Parameters<'a> {
     #[cfg(feature = "scrypt")]
     #[cfg_attr(docsrs, doc(cfg(feature = "scrypt")))]
     pub fn scrypt_aes128cbc(
-        scrypt_log_n: u8,
-        scrypt_r: u32,
-        scrypt_p: u32,
-        scrypt_salt: &'a [u8],
+        params: scrypt::Params,
+        salt: &'a [u8],
         aes_iv: &'a [u8; AES_BLOCK_SIZE],
     ) -> Result<Self, CryptoError> {
-        let kdf = ScryptParams::from_params_and_salt(
-            scrypt::Params::new(scrypt_log_n, scrypt_r, scrypt_p).map_err(|_| CryptoError)?,
-            scrypt_salt,
-        )?
-        .into();
+        let kdf = ScryptParams::from_params_and_salt(params, salt)?.into();
         let encryption = EncryptionScheme::Aes128Cbc { iv: aes_iv };
         Ok(Self { kdf, encryption })
     }
@@ -106,20 +121,17 @@ impl<'a> Parameters<'a> {
     ///
     /// For more information on scrypt parameters, see documentation for the
     /// [`scrypt::Params`] struct.
+    ///
+    /// When in doubt, use `Default::default()` as the [`scrypt::Params`].
+    /// This also avoids the need to import the type from the `scrypt` crate.
     #[cfg(feature = "scrypt")]
     #[cfg_attr(docsrs, doc(cfg(feature = "scrypt")))]
     pub fn scrypt_aes256cbc(
-        scrypt_log_n: u8,
-        scrypt_r: u32,
-        scrypt_p: u32,
-        scrypt_salt: &'a [u8],
+        params: scrypt::Params,
+        salt: &'a [u8],
         aes_iv: &'a [u8; AES_BLOCK_SIZE],
     ) -> Result<Self, CryptoError> {
-        let kdf = ScryptParams::from_params_and_salt(
-            scrypt::Params::new(scrypt_log_n, scrypt_r, scrypt_p).map_err(|_| CryptoError)?,
-            scrypt_salt,
-        )?
-        .into();
+        let kdf = ScryptParams::from_params_and_salt(params, salt)?.into();
         let encryption = EncryptionScheme::Aes256Cbc { iv: aes_iv };
         Ok(Self { kdf, encryption })
     }
@@ -229,11 +241,31 @@ pub enum EncryptionScheme<'a> {
         iv: &'a [u8; AES_BLOCK_SIZE],
     },
 
+    /// AES-192 in CBC mode
+    Aes192Cbc {
+        /// Initialization vector
+        iv: &'a [u8; AES_BLOCK_SIZE],
+    },
+
     /// AES-256 in CBC mode
     Aes256Cbc {
         /// Initialization vector
         iv: &'a [u8; AES_BLOCK_SIZE],
     },
+
+    /// 3-Key Triple DES in CBC mode
+    #[cfg(feature = "3des")]
+    DesEde3Cbc {
+        /// Intialisation vector
+        iv: &'a [u8; DES_BLOCK_SIZE],
+    },
+
+    /// DES in CBC mode
+    #[cfg(feature = "des-insecure")]
+    DesCbc {
+        /// Initialisation vector
+        iv: &'a [u8; DES_BLOCK_SIZE],
+    },
 }
 
 impl<'a> EncryptionScheme<'a> {
@@ -241,7 +273,12 @@ impl<'a> EncryptionScheme<'a> {
     pub fn key_size(&self) -> usize {
         match self {
             Self::Aes128Cbc { .. } => 16,
+            Self::Aes192Cbc { .. } => 24,
             Self::Aes256Cbc { .. } => 32,
+            #[cfg(feature = "des-insecure")]
+            Self::DesCbc { .. } => 8,
+            #[cfg(feature = "3des")]
+            Self::DesEde3Cbc { .. } => 24,
         }
     }
 
@@ -249,7 +286,12 @@ impl<'a> EncryptionScheme<'a> {
     pub fn oid(&self) -> ObjectIdentifier {
         match self {
             Self::Aes128Cbc { .. } => AES_128_CBC_OID,
+            Self::Aes192Cbc { .. } => AES_192_CBC_OID,
             Self::Aes256Cbc { .. } => AES_256_CBC_OID,
+            #[cfg(feature = "des-insecure")]
+            Self::DesCbc { .. } => DES_CBC_OID,
+            #[cfg(feature = "3des")]
+            Self::DesEde3Cbc { .. } => DES_EDE3_CBC_OID,
         }
     }
 }
@@ -267,18 +309,36 @@ impl<'a> TryFrom<AlgorithmIdentifier<'a>> for EncryptionScheme<'a> {
 
     fn try_from(alg: AlgorithmIdentifier<'a>) -> der::Result<Self> {
         // TODO(tarcieri): support for non-AES algorithms?
-        let iv = alg
-            .parameters_any()?
-            .octet_string()?
-            .as_bytes()
-            .try_into()
-            .map_err(|_| ErrorKind::Value {
-                tag: der::Tag::OctetString,
-            })?;
+        let iv = alg.parameters_any()?.octet_string()?.as_bytes();
 
         match alg.oid {
-            AES_128_CBC_OID => Ok(Self::Aes128Cbc { iv }),
-            AES_256_CBC_OID => Ok(Self::Aes256Cbc { iv }),
+            AES_128_CBC_OID => Ok(Self::Aes128Cbc {
+                iv: iv
+                    .try_into()
+                    .map_err(|_| der::Tag::OctetString.value_error())?,
+            }),
+            AES_192_CBC_OID => Ok(Self::Aes192Cbc {
+                iv: iv
+                    .try_into()
+                    .map_err(|_| der::Tag::OctetString.value_error())?,
+            }),
+            AES_256_CBC_OID => Ok(Self::Aes256Cbc {
+                iv: iv
+                    .try_into()
+                    .map_err(|_| der::Tag::OctetString.value_error())?,
+            }),
+            #[cfg(feature = "des-insecure")]
+            DES_CBC_OID => Ok(Self::DesCbc {
+                iv: iv[0..DES_BLOCK_SIZE]
+                    .try_into()
+                    .map_err(|_| der::Tag::OctetString.value_error())?,
+            }),
+            #[cfg(feature = "3des")]
+            DES_EDE3_CBC_OID => Ok(Self::DesEde3Cbc {
+                iv: iv[0..DES_BLOCK_SIZE]
+                    .try_into()
+                    .map_err(|_| der::Tag::OctetString.value_error())?,
+            }),
             oid => Err(ErrorKind::UnknownOid { oid }.into()),
         }
     }
@@ -290,7 +350,12 @@ impl<'a> TryFrom<EncryptionScheme<'a>> for AlgorithmIdentifier<'a> {
     fn try_from(scheme: EncryptionScheme<'a>) -> der::Result<Self> {
         let parameters = OctetString::new(match scheme {
             EncryptionScheme::Aes128Cbc { iv } => iv,
+            EncryptionScheme::Aes192Cbc { iv } => iv,
             EncryptionScheme::Aes256Cbc { iv } => iv,
+            #[cfg(feature = "des-insecure")]
+            EncryptionScheme::DesCbc { iv } => iv,
+            #[cfg(feature = "3des")]
+            EncryptionScheme::DesEde3Cbc { iv } => iv,
         })?;
 
         Ok(AlgorithmIdentifier {
